@@ -1,5 +1,5 @@
 /*
- * $Id: worker.h 6162 2012-02-13 12:33:26Z jerry $
+ * $Id: worker.h 7124 2013-05-03 09:49:26Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -67,6 +67,7 @@ struct worker_struct {
     int need_to_exit;
     cond_basic_type worker_alarm;
     lock_basic_type worker_lock;
+    int worker_locked;
 };
 
 /**
@@ -121,6 +122,14 @@ void worker_wakeup(worker_type* worker);
  */
 void worker_wait(lock_basic_type* lock, cond_basic_type* condition);
 
+/**
+ * Let worker wait on an already locked cond.
+ * \param[in] lock lock to use
+ * \param[in] condition condition to be met
+ *
+ */
+void worker_wait_locked(lock_basic_type* lock, cond_basic_type* condition);
+
 /**
  * Let worker wait.
  * \param[in] lock lock to use
@@ -132,7 +141,7 @@ void worker_wait_timeout(lock_basic_type* lock, cond_basic_type* condition,
     time_t timeout);
 
 /**
- * Let worker wait on an already locked cond
+ * Let worker wait on an already locked cond.
  * \param[in] lock lock to use
  * \param[in] condition condition to be met
  * \param[in] timeout maximum waiting time in seconds
