@@ -1,5 +1,5 @@
 /*
- * $Id: file.c 6244 2012-04-03 13:56:27Z matthijs $
+ * $Id: file.c 6498 2012-08-02 21:56:06Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -90,6 +90,10 @@ ods_fgetc(FILE* fd, unsigned int* line_nr)
     if (c == '\n') {
         (*line_nr)++;
     }
+    if (c == EOF && errno != 0) {
+        ods_log_error("[%s] fgetc() failed, enough memory? (%s)",
+            file_str, strerror(errno));
+    }
     return c;
 }
 
@@ -204,6 +208,9 @@ ods_fopen(const char* file, const char* dir, const char* mode)
     if (len_total > 0) {
         openf = (char*) malloc(sizeof(char)*(len_total + 1));
         if (!openf) {
+            ods_log_error("[%s] unable to open file %s%s%s for %s: malloc() "
+                "failed", file_str, (dir?dir:""), (dir?"/":""),
+                (file?file:"(null)"), ods_file_mode2str(mode));
             return NULL;
         }
         if (dir) {
@@ -220,7 +227,7 @@ ods_fopen(const char* file, const char* dir, const char* mode)
         if (len_file) {
             fd = fopen(openf, mode);
             if (!fd) {
-                ods_log_verbose("[%s] unable to open file %s for %s: %s",
+                ods_log_error("[%s] unable to open file %s for %s: %s",
                     file_str, openf?openf:"(null)",
                     ods_file_mode2str(mode), strerror(errno));
             }
@@ -288,7 +295,15 @@ ods_file_lastmodified(const char* file)
     if ((fd = ods_fopen(file, NULL, "r")) != NULL) {
         ret = stat(file, &buf);
         ods_fclose(fd);
+        if (ret == -1) {
+            ods_log_error("[%s] unable to stat file %s: %s",
+                    file_str, file, strerror(errno));
+            return 0;
+        }
         return buf.st_mtime;
+    } else {
+        ods_log_error("[%s] unable to stat file %s: ods_fopen() failed",
+            file_str, file, strerror(errno));
     }
     return 0;
 }
