@@ -1,5 +1,5 @@
 /*
- * $Id: util.c 4341 2011-01-31 15:21:09Z matthijs $
+ * $Id: util.c 6956 2013-01-17 10:57:36Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -36,8 +36,13 @@
 #include "shared/log.h"
 #include "shared/util.h"
 
-#include <time.h>
+#include <fcntl.h>
 #include <ldns/ldns.h>
+#include <signal.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <time.h>
+#include <unistd.h>
 
 static const char* util_str = "util";
 
@@ -234,6 +239,94 @@ util_dnssec_rrs_add_rr(ldns_dnssec_rrs *rrs, ldns_rr *rr)
 }
 
 
+/**
+ * Read process id from file.
+ *
+ */
+static pid_t
+util_read_pidfile(const char* file)
+{
+    int fd;
+    pid_t pid;
+    char pidbuf[32];
+    char *t;
+    int l;
+
+    if ((fd = open(file, O_RDONLY)) == -1) {
+        return -1;
+    }
+    if (((l = read(fd, pidbuf, sizeof(pidbuf)))) == -1) {
+        close(fd);
+        return -1;
+    }
+    close(fd);
+    /* Empty pidfile means no pidfile... */
+    if (l == 0) {
+        errno = ENOENT;
+        return -1;
+    }
+    pid = (pid_t) strtol(pidbuf, &t, 10);
+
+    if (*t && *t != '\n') {
+        return -1;
+    }
+    return pid;
+}
+
+
+/**
+ * Check process id file.
+ *
+ */
+int
+util_check_pidfile(const char* pidfile)
+{
+    pid_t oldpid;
+    struct stat stat_ret;
+
+    if (!pidfile) {
+        return 1;
+    }
+    /**
+     * If the file exists then either we didn't shutdown cleanly or
+     * a signer daemon is already running: in either case shutdown.
+     */
+    if (stat(pidfile, &stat_ret) != 0) {
+        if (errno != ENOENT) {
+            ods_log_error("[%s] cannot stat pidfile %s: %s", util_str, pidfile,
+                strerror(errno));
+        } /* else: file does not exist: carry on */
+    } else {
+          if (S_ISREG(stat_ret.st_mode)) {
+            /** The pidfile exists already */
+            if ((oldpid = util_read_pidfile(pidfile)) == -1) {
+                /** Consider stale pidfile */
+                if (errno != ENOENT) {
+                    ods_log_error("[%s] cannot read pidfile %s: %s", util_str,
+                        pidfile, strerror(errno));
+                }
+            } else {
+                if (kill(oldpid, 0) == 0 || errno == EPERM) {
+                    ods_log_crit("[%s] pidfile %s already exists, "
+                        "a process with pid %u is already running. "
+                        "If no ods-signerd process is running, a previous "
+                        "instance didn't shutdown cleanly, please remove this "
+                        "file and try again.", util_str, pidfile, oldpid);
+                    return 0;
+                } else {
+                    /** Consider state pidfile */
+                    ods_log_warning("[%s] pidfile %s already exists, "
+                        "but no process with pid %u is running. "
+                        "A previous instance didn't shutdown cleanly, this "
+                        "pidfile is stale.", util_str, pidfile, oldpid);
+                }
+            }
+        }
+    }
+    /** All good, carry on */
+    return 1;
+}
+
 /**
  * Write process id to file.
  *
@@ -245,8 +338,9 @@ util_write_pidfile(const char* pidfile, pid_t pid)
     char pidbuf[32];
     size_t result = 0, size = 0;
 
-    ods_log_assert(pidfile);
-    ods_log_assert(pid);
+    if (!pidfile || !pid) {
+        return -1;
+    }
     ods_log_debug("[%s] writing pid %lu to pidfile %s", util_str,
         (unsigned long) pid, pidfile);
     snprintf(pidbuf, sizeof(pidbuf), "%lu\n", (unsigned long) pid);
